from ftw.events import _
from ftw.events.interfaces import IEventListingView
from plone import api
from plone.app.event.dx.behaviors import IEventLocation
from Products.CMFPlone.PloneBatch import Batch
from Products.Five.browser import BrowserView
from zope.component import getMultiAdapter
from zope.contentprovider.interfaces import IContentProvider
from zope.i18n import translate
from zope.interface import implements


class EventListing(BrowserView):
    """
    This browser view renders a list of event pages based on the parameters
    defined on the event listing block which renders a link to this browser
    view.
    """
    implements(IEventListingView)

    def __init__(self, context, request):
        super(EventListing, self).__init__(context, request)
        self.batch_size = 10

    @property
    def batch(self):
        b_start = self.request.form.get('b_start', 0)
        return Batch(self.get_items(), self.batch_size, b_start)

    def get_query(self):
        block_view = api.content.get_view(
            name='block_view',
            context=self.context,
            request=self.request,
        )
        block_query = block_view.get_query()
        return block_query

    def get_items(self):
        """
        Returns a list of event page brains.
        """
        catalog = api.portal.get_tool(name='portal_catalog')
        return catalog(self.get_query())

    def get_event_page_dict(self, brain):
        """
        Constructs a dict containing the data of the brain of an event page.
        """
        obj = brain.getObject()

        # Copied from "plone.app.event.portlets.portlet_events.Renderer#formatted_date".
        provider = getMultiAdapter(
            (self.context, self.request, self),
            IContentProvider, name='formatted_date'
        )
        date_snippet = provider(obj)

        location = ''
        if IEventLocation(obj, None):
            location = obj.location

        item = {
            'title': brain.Title,
            'description': brain.Description,
            'url': brain.getURL(),
            'brain': brain,
            'date_snippet': date_snippet,
            'location': location,
        }
        return item

    @property
    def title(self):
        """
        Returns the title defined on the event listing block or a
        fallback title.
        """
        fallback_title = translate(
            _('more_items_view_fallback_title', default=u'Events'),
            context=self.request
        )
        return self.context.more_items_view_title or fallback_title



from ftw.builder.testing import BUILDER_LAYER
from ftw.builder.testing import functional_session_factory
from ftw.builder.testing import set_builder_session_factory
from plone.app.testing import applyProfile
from plone.app.testing import FunctionalTesting
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import PLONE_ZSERVER
from plone.app.testing import PloneSandboxLayer
from plone.testing import z2
from zope.configuration import xmlconfig
import ftw.events.tests.builders


class EventsLayer(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE, BUILDER_LAYER)

    def setUpZope(self, app, configurationContext):
        xmlconfig.string(
            '<configure xmlns="http://namespaces.zope.org/zope">'
            '  <include package="z3c.autoinclude" file="meta.zcml" />'
            '  <includePlugins package="plone" />'
            '  <includePluginsOverrides package="plone" />'
            '</configure>',
            context=configurationContext)

        z2.installProduct(app, 'Products.DateRecurringIndex')
        z2.installProduct(app, 'ftw.simplelayout')

    def setUpPloneSite(self, portal):
        applyProfile(portal, 'ftw.events:default')


EVENTS_FIXTURE = EventsLayer()
FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(EVENTS_FIXTURE,
           set_builder_session_factory(functional_session_factory)),
    name="ftw.events:functional")


class EventsZserverLayer(PloneSandboxLayer):

    defaultBases = (EVENTS_FIXTURE, PLONE_ZSERVER)


EVENTS_ZSERVER_FIXTURE = EventsZserverLayer()
FUNCTIONAL_ZSERVER_TESTING = FunctionalTesting(
    bases=(EVENTS_ZSERVER_FIXTURE,
           set_builder_session_factory(functional_session_factory)),
    name="ftw.events:zserverfunctional")



from Products.CMFCore.utils import getToolByName


def default_profile_installed(portal):
    set_calendar_types(portal)
    reindex_indexes(portal)


def set_calendar_types(context):
    """
    Mark `ftw.events.EventPage` as an calendar type in portal_calendar.
    """
    portal_calendar = getToolByName(context, 'portal_calendar')
    types = list(portal_calendar.calendar_types)
    types.append('ftw.events.EventPage')
    portal_calendar.calendar_types = tuple(types)


def reindex_indexes(portal):
    """
    `plone.app.events` replaces the start and end index by a
    RecurringDateIndex (before DateIndex) but it omits to reindex
    the new index. This causes the loss of the start and end index
    data of every content. So we need to take care of reindexing
    the indexes.
    """
    catalog = getToolByName(portal, 'portal_catalog')
    catalog.reindexIndex('start', None)
    catalog.reindexIndex('end', None)



from ftw.events.interfaces import IEventPage
from plone.app.event.dx.behaviors import IEventBasic
from plone.autoform.interfaces import IFormFieldProvider
from plone.dexterity.content import Container
from plone.directives import form
from plone.supermodel.interfaces import FIELDSETS_KEY
from zope.interface import alsoProvides
from zope.interface import implements


# Move plone.app.event's default field "timezone" to the default schema,
# since all other fields are in the default schema and we do not want
# an extra schema only for the "timezone" field.
for fieldset in IEventBasic.queryTaggedValue(FIELDSETS_KEY, []):
    if 'timezone' in fieldset.fields:
        fieldset.fields.remove('timezone')


class IEventPageSchema(form.Schema):
    pass

alsoProvides(IEventPageSchema, IFormFieldProvider)


class EventPage(Container):
    implements(IEventPage)



from ftw.events import _
from ftw.events.interfaces import IEventListingBlock
from plone.autoform.interfaces import IFormFieldProvider
from plone.dexterity.content import Item
from plone.directives import form
from plone.formwidget.autocomplete.widget import AutocompleteMultiFieldWidget
from plone.formwidget.contenttree import MultiContentTreeFieldWidget
from plone.formwidget.contenttree import ObjPathSourceBinder
from Products.CMFPlone.interfaces.syndication import IFeedSettings
from Products.CMFPlone.interfaces.syndication import ISyndicatable
from z3c.relationfield import RelationChoice
from zope import schema
from zope.interface import alsoProvides
from zope.interface import implements
from zope.interface import invariant, Invalid


class IEventListingBlockSchema(form.Schema):
    title = schema.TextLine(
        title=_(u'event_listing_config_title_label', default=u'Title'),
        description=u'',
        required=True,
        default=u'',
    )

    show_title = schema.Bool(
        title=_(u'event_listing_block_show_title_label', default=u'Show title'),
        default=True,
        required=False,
    )

    exclude_past_events = schema.Bool(
        title=_(u'label_exclude_past_events', default=u'Exclude past events'),
        default=True,
        required=False,
    )

    form.widget(filter_by_path=MultiContentTreeFieldWidget)
    filter_by_path = schema.List(
        title=_(u'event_listing_config_filter_path_label',
                default=u'Limit to path'),
        description=_(u'event_listing_config_filter_path_description',
                      default=u'Only show content from a specific path.'),
        value_type=RelationChoice(
            source=ObjPathSourceBinder(
                navigation_tree_query={'is_folderish': True},
                is_folderish=True
            ),
        ),
        required=False,
        default=[],
        missing_value=[],
    )

    current_context = schema.Bool(
        title=_(u'event_listing_config_filter_current_context_label',
                default=u'Limit to current context'),
        description=_(
            u'event_listing_config_filter_current_context_description',
            default=u'Only show items from the current context.'),
        default=True,
    )

    quantity = schema.Int(
        title=_(u'event_listing_config_quantity_label', default=u'Quantity'),
        description=_(u'event_listing_config_quantity_description',
                      default=u'The number of items to be shown at most. '
                              u'Enter 0 for no limitation.'),
        default=5,
    )

    # MAYBE: Find a better widget.
    form.widget(subjects=AutocompleteMultiFieldWidget)
    subjects = schema.List(
        title=_(u'event_listing_config_subjects_label',
                default=u'Filter by subject'),
        description=_(u'event_listing_config_subjects_description',
                      default=u'Only items with the selected subjects will '
                              u'be shown.'),
        value_type=schema.Choice(vocabulary='ftw.events.vocabulary.subjects'),
        required=False,
        default=[],
        missing_value=[],
    )

    show_description = schema.Bool(
        title=_(u'event_listing_config_show_description_label',
                default=u'Show the description of the item'),
        default=True,
    )

    description_length = schema.Int(
        title=_(u'event_listing_config_description_length_label',
                default=u'Length of the description'),
        description=_(u'event_listing_config_description_length_description',
                      default=u'The maximum length of the item\'s '
                              u'description. Longer descriptions will be '
                              u'cropped. Enter 0 for no limitation.'),
        default=50,
    )

    show_more_items_link = schema.Bool(
        title=_(u'event_listing_config_show_more_items_link',
                default=u'Show link to more items'),
        description=_(u'event_listing_show_more_items_link_description',
                      default=u'Render a link to a page which renders more '
                              u'items (only if there is at least one item).'),
        default=False,
    )

    more_items_link_label = schema.TextLine(
        title=_(u'label_more_items_link_label',
                default=u'Label for the "more items" link'),
        description=_(u'description_more_items_link_label',
                      default=u'This custom label will not be translated.'),
        required=False,
    )

    more_items_view_title = schema.TextLine(
        title=_(u'label_more_items_view_title',
                default=u'Title of the view behind the "more items" link'),
        description=_(u'description_more_items_view_title',
                      default=u'This title will not be translated.'),
        required=False,
    )

    @invariant
    def is_either_path_or_context(obj):
        """Checks if not both path and current context are defined.
        """
        if obj.current_context and obj.filter_by_path:
            raise Invalid(_(
                u'event_listing_config_current_context_and_path_error',
                default=u'You cannot filter by path and current context '
                        u'at the same time.')
            )

alsoProvides(IEventListingBlockSchema, IFormFieldProvider)


class EventListingBlock(Item):
    implements(IEventListingBlock, ISyndicatable)


def enable_syndication(event_listing_block, event=None):
    """
    Enables syndication on the given event listing block. This
    must be called from a subscriber when an event listing block is
    created so the syndication is enabled by default on the event
    listing blocks.
    """
    settings = IFeedSettings(event_listing_block)
    settings.enabled = True
    settings.feed_types = ('RSS', 'rss.xml', 'atom.xml')



from ftw.builder import builder_registry
from ftw.builder.dexterity import DexterityBuilder
from ftw.simplelayout.tests import builders


class EventFolderBuilder(DexterityBuilder):
    portal_type = 'ftw.events.EventFolder'

builder_registry.register('event folder', EventFolderBuilder)


class EventPageBuilder(DexterityBuilder):
    portal_type = 'ftw.events.EventPage'

    def __init__(self, *args, **kwargs):
        super(EventPageBuilder, self).__init__(*args, **kwargs)
        # ftw.builders auto-discovery does not work for recurrence:
        self.having(recurrence='')
        # using greenwich, since the test-site / browser is set up with it:
        self.having(timezone='Etc/Greenwich')

    def starting(self, date):
        return self.having(start=date)

    def ending(self, date):
        return self.having(end=date)

builder_registry.register('event page', EventPageBuilder)


class EventListingBlockBuilder(DexterityBuilder):
    portal_type = 'ftw.events.EventListingBlock'

builder_registry.register('event listing block', EventListingBlockBuilder)



from ftw.builder import Builder
from ftw.builder import create
from ftw.events.tests import FunctionalTestCase
from ftw.testbrowser import browsing
from Products.CMFPlone.interfaces.syndication import IFeedSettings
import transaction


class TestSyndication(FunctionalTestCase):

    def setUp(self):
        super(TestSyndication, self).setUp()
        self.grant('Manager')

    @browsing
    def test_feed_only_renders_event_pages(self, browser):
        """
        This test makes sure that the standard Plone syndication machinery
        only renders event pages (and not envent listing blocks available
        on the event folder).
        """
        event_folder = create(Builder('event folder')
                              .titled(u'Not relevant in this test'))
        create(Builder('event page')
               .titled(u'My Event')
               .within(event_folder))
        create(Builder('event page')
               .titled(u'Another Event')
               .within(event_folder))
        create(Builder('event listing block')
               .within(event_folder)
               .titled(u'Event listing block'))

        # Enable syndication on the event folder.
        IFeedSettings(event_folder).enabled = True
        transaction.commit()

        browser.login()

        # Make sure only event pages are rendered.
        browser.visit(event_folder, view='atom.xml')
        self.assertEqual(
            ['My Event', 'Another Event'],
            browser.css('entry title').text
        )



from Products.CMFPlone.browser.syndication.adapters import FolderFeed
from plone import api
from ftw.events.interfaces import IEventPage


class EventFolderFeed(FolderFeed):

    def _brains(self):
        catalog = api.portal.get_tool('portal_catalog')
        query = {
            'object_provides': IEventPage.__identifier__,
            'path': {
                'query': '/'.join(self.context.getPhysicalPath()),
            },
            'sort_on': 'start',
            'sort_order': 'ascending',
        }
        return catalog(**query)


class EventListingBlockFeed(FolderFeed):

    def _brains(self):
        catalog = api.portal.get_tool('portal_catalog')
        query = self.get_query()
        return catalog(**query)

    def get_query(self):
        block_view = api.content.get_view(
            name='block_view',
            context=self.context,
            request=self.context.REQUEST,
        )
        block_query = block_view.get_query()
        return block_query



from Acquisition._Acquisition import aq_inner, aq_parent
from ftw.events import _
from ftw.events import utils
from ftw.events.interfaces import IEventPage
from ftw.simplelayout.browser.blocks.base import BaseBlock
from plone.app.event.base import _prepare_range, filter_and_resort
from plone.app.event.dx.behaviors import IEventLocation
from Products.CMFCore.utils import getToolByName
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope.component import getMultiAdapter
from zope.contentprovider.interfaces import IContentProvider
from zope.i18n import translate
import datetime


class EventListingBlockView(BaseBlock):
    template = ViewPageTemplateFile('templates/eventlistingblock.pt')

    def get_block_info(self):
        """
        This method returns a dict containing information to be used in
        the block's template.
        """
        more_items_link_url = ''
        if self.context.show_more_items_link:
            more_items_link_url = '/'.join([self.context.absolute_url(), 'events'])

        more_items_link_label = (
            self.context.more_items_link_label or
            translate(_('more_items_link_label', default=u'More Items'),
                      context=self.request)
        )

        info = {
            'title': self.context.title,
            'show_title': self.context.show_title,
            'more_items_link_url': more_items_link_url,
            'more_items_link_label': more_items_link_label,
        }

        return info

    def get_items(self):
        """
        Returns a list of dicts where each dict contains the data of an
        event page. It is called from the template.
        """
        catalog = getToolByName(self.context, 'portal_catalog')

        start = None
        end = None

        if self.context.exclude_past_events:
            # Start from midnight of today. This way the query will also include
            # events which have ended just a few hours ago.
            midnight_of_today = datetime.datetime.now().replace(
                hour=0, minute=0, second=0, microsecond=0
            )
            start = midnight_of_today

        # Inspired by `plone.app.event.base.get_events`.
        start, end = _prepare_range(self.context, start, end)

        brains = catalog.searchResults(
            self.get_query(start, end)
        )

        # Inspired by `plone.app.event.base.get_events`.
        brains = filter_and_resort(
            context=self.context, brains=brains, start=start,
            end=end, sort='start', sort_reverse=False
        )

        if self.context.quantity:
            brains = brains[:self.context.quantity]

        items = [self.get_event_page_dict(brain) for brain in brains]

        return items

    def get_query(self, start=None, end=None):
        query = {
            'object_provides': IEventPage.__identifier__,
            'sort_on': 'start',
        }

        # Inspired by `plone.app.event.base.get_events`.
        if start:
            # All events from start date ongoing:
            # The minimum end date of events is the date from which we search.
            query['end'] = {'query': start, 'range': 'min'}
        if end:
            # All events until end date:
            # The maximum start date must be the date until we search.
            query['start'] = {'query': end, 'range': 'max'}

        parent = aq_parent(aq_inner(self.context))
        if self.context.current_context:
            # `parent` is the object containing this block.
            path = '/'.join(parent.getPhysicalPath())
            query['path'] = {'query': path}
        elif self.context.filter_by_path:
            paths = ['/'.join(item.getPhysicalPath()) for item in self.context.filter_by_path]
            query['path'] = {'query': paths}

        if self.context.subjects:
            query['Subject'] = self.context.subjects

        return query

    def get_event_page_dict(self, brain):
        """
        Constructs a dict containing the data of the brain of an event page.
        """
        obj = brain.getObject()

        description = ''
        if self.context.show_description:
            description = brain.Description
        if self.context.description_length:
            description = utils.crop_text(
                description, self.context.description_length
            )

        # Copied from "plone.app.event.portlets.portlet_events.Renderer#formatted_date".
        provider = getMultiAdapter(
            (self.context, self.request, self),
            IContentProvider, name='formatted_date'
        )
        date_snippet = provider(obj)

        location = ''
        if IEventLocation(obj, None):
            location = obj.location

        item = {
            'title': brain.Title,
            'description': description,
            'url': brain.getURL(),
            'brain': brain,
            'date_snippet': date_snippet,
            'location': location,
        }
        return item



from zope.interface import Interface


class IFTWEventsLayer(Interface):
    """Request layer for ftw.events"""


class IEventFolder(Interface):
    """Event folder marker interface.
    """


class IEventPage(Interface):
    """Event page folder marker interface.
    """


class IEventListingBlock(Interface):
    """Marker interface for the event listing blocks
    """


class IEventListingView(Interface):
    """Marker interface for the event listing view
    """



from datetime import datetime
from ftw.builder import Builder
from ftw.builder import create
from ftw.events.tests import RealFuncitionalTestCase
from ftw.testbrowser import browsing
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import TEST_USER_PASSWORD
import requests
from ftw.testbrowser.core import LIB_REQUESTS


class TestIsICSView(RealFuncitionalTestCase):

    def setUp(self):
        self.portal = self.layer['portal']
        self.grant('Manager')
        self.folder = create(Builder('event folder').titled(u'Activities'))

        self.event = create(Builder('event page')
                            .titled(u"A Event")
                            .starting(datetime(2013, 10, 7, 9, 00))
                            .ending(datetime(2013, 10, 7, 16, 00))
                            .having(recurrence='RRULE:FREQ=DAILY;COUNT=4')
                            .within(self.folder))

    @browsing
    def test_ics_view_is_ics(self, browser):
        browser.request_library = LIB_REQUESTS
        browser.login().visit(self.event, view="ics_view")

        response = browser.response
        body = response.content
        self.assertIn('SUMMARY:A Event', body)
        self.assertIn('BEGIN:VCALENDAR', body)
        self.assertIn('BEGIN:VEVENT', body)
        self.assertIn('END:VCALENDAR', body)
        self.assertIn('END:VEVENT', body)



from ftw.builder import Builder
from ftw.builder import create
from ftw.events.tests import FunctionalTestCase
from ftw.testbrowser import browsing
from ftw.testbrowser.pages import factoriesmenu
from ftw.testbrowser.pages import statusmessages
from Products.CMFPlone.interfaces.syndication import IFeedSettings
import transaction


class TestEventFolder(FunctionalTestCase):

    @browsing
    def test_create_event_folder(self, browser):
        self.grant('Manager')
        browser.login().open()
        factoriesmenu.add('Event Folder')

        browser.fill({'Title': 'Activities'}).save()
        statusmessages.assert_no_error_messages()
        self.assertEquals('http://nohost/plone/activities/view', browser.url)

    @browsing
    def test_event_folder_has_empty_listing_block(self, browser):
        self.grant('Manager')
        folder = create(Builder('event folder'))
        browser.login().visit(folder, view='@@simplelayout-view')
        self.assertEqual(
            ['No content available.'],
            browser.css('.sl-layout .ftw-events-eventlistingblock').text
        )

    @browsing
    def test_event_folder_rss_feed_only_renders_event_pages(self, browser):
        self.grant('Manager')
        folder = create(Builder('event folder'))
        event1 = create(Builder('event page').titled(u'Event 1').within(folder))
        event2 = create(Builder('event page').titled(u'Event 2').within(folder))

        IFeedSettings(folder).enabled = True
        transaction.commit()

        # Make sure the event folder contains the event listing block
        # and two event pages.
        self.assertEqual(
            ['ftw.events.EventListingBlock', 'ftw.events.EventPage', 'ftw.events.EventPage'],
            [obj.portal_type for obj in folder.listFolderContents()]
        )

        # Make sure that the RSS feed only renders the event pages,
        # not the event listing block.
        browser.login().visit(folder, view='rss.xml')
        self.assertEqual(
            [event1.Title(), event2.Title()],
            browser.css('item title').text
        )

    @browsing
    def test_event_folder_contains_links_to_rss_feeds_of_block(self, browser):
        """
        This test makes sure that the feed links of the event listing block
        (which is created automatically after the creation of the event folder)
         are rendered in the source of the event folder.
        """
        self.grant('Manager')
        folder = create(Builder('event folder'))
        browser.login()

        browser.visit(folder)
        self.assertEqual(
            [
                (
                    'application/rss+xml',
                    'http://nohost/plone/ftw-events-eventfolder/events/RSS',
                    'Events - RSS 1.0'
                ),
                (
                    'application/rss+xml',
                    'http://nohost/plone/ftw-events-eventfolder/events/rss.xml',
                    'Events - RSS 2.0'
                ),
                (
                    'application/rss+xml',
                    'http://nohost/plone/ftw-events-eventfolder/events/atom.xml',
                    'Events - Atom'
                )
            ],
            [
                (link.attrib['type'], link.attrib['href'], link.attrib['title'])
                for link in browser.css('link[rel="alternate"]')
            ]
        )

        # Disable syndication on the event listing block and make sure
        # the feed links are no longer rendered in the source of the
        # event folder.
        # Note that a default event listing block is created
        # automatically in the event folder.
        block = folder.listFolderContents(
            contentFilter={'portal_type': 'ftw.events.EventListingBlock'}
        )[0]
        IFeedSettings(block).enabled = False
        transaction.commit()

        browser.visit(folder)
        self.assertEqual([], browser.css('link[rel="alternate"]'))

    @browsing
    def test_event_folder_contains_links_to_rss_feeds_of_multiple_blocks(self, browser):
        """
        This test makes sure that the feed links of multiples event listing blocks
        are rendered in the source of the event folder.
        """
        self.grant('Manager')
        folder = create(Builder('event folder'))
        create(Builder('event listing block')
               .titled(u'Event Listing Block 2')
               .within(folder))
        browser.login()

        browser.visit(folder)
        self.assertEqual(
            [
                (
                    'application/rss+xml',
                    'http://nohost/plone/ftw-events-eventfolder/events/RSS',
                    'Events - RSS 1.0'
                ),
                (
                    'application/rss+xml',
                    'http://nohost/plone/ftw-events-eventfolder/events/rss.xml',
                    'Events - RSS 2.0'
                ),
                (
                    'application/rss+xml',
                    'http://nohost/plone/ftw-events-eventfolder/events/atom.xml',
                    'Events - Atom'
                ),
                (
                    'application/rss+xml',
                    'http://nohost/plone/ftw-events-eventfolder/event-listing-block-2/RSS',
                    'Event Listing Block 2 - RSS 1.0'
                ),
                (
                    'application/rss+xml',
                    'http://nohost/plone/ftw-events-eventfolder/event-listing-block-2/rss.xml',
                    'Event Listing Block 2 - RSS 2.0'
                ),
                (
                    'application/rss+xml',
                    'http://nohost/plone/ftw-events-eventfolder/event-listing-block-2/atom.xml',
                    'Event Listing Block 2 - Atom'
                )
            ],
            [
                (link.attrib['type'], link.attrib['href'], link.attrib['title'])
                for link in browser.css('link[rel="alternate"]')
            ]
        )



from setuptools import setup, find_packages
import os

version = '1.0.1'

tests_require = [
    'ftw.builder',
    'ftw.lawgiver',
    'ftw.testbrowser',
    'ftw.testing',
    'plone.app.testing',
    'plone.testing',
]

extras_require = {
    'tests': tests_require,
    'development': [
        'plonetheme.blueberry',
    ]
}

setup(
    name='ftw.events',
    version=version,
    description='Events with simplelayout.',
    long_description=open('README.rst').read() + '\n' + open(
        os.path.join('docs', 'HISTORY.txt')).read(),

    classifiers=[
        'Framework :: Plone',
        'Framework :: Plone :: 4.3',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],

    keywords='ftw events',
    author='4teamwork AG',
    author_email='mailto:info@4teamwork.ch',
    url='https://github.com/4teamwork/ftw.events',
    license='GPL2',

    packages=find_packages(exclude=['ez_setup']),
    namespace_packages=['ftw', ],
    include_package_data=True,
    zip_safe=False,

    install_requires=[
        'Plone',
        'collective.dexteritytextindexer',
        'ftw.profilehook',
        'ftw.simplelayout [contenttypes]',
        'ftw.upgrade',
        'plone.api',
        'plone.app.dexterity',
        'plone.app.event [dexterity]',
        'plone.app.referenceablebehavior',
        'plone.directives.form',
        'setuptools',
    ],

    tests_require=tests_require,
    extras_require=extras_require,

    entry_points="""
    # -*- Entry points: -*-
    [z3c.autoinclude.plugin]
    target = plone
    """)



from hashlib import md5
from plone import api
from plone.i18n.normalizer.interfaces import IIDNormalizer
from zope.component import queryUtility
from zope.interface import directlyProvides
from zope.schema.interfaces import IVocabularyFactory
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary


class SubjectVocabulary(SimpleVocabulary):

    def __init__(self, terms, *interfaces):
        terms = self.get_terms()
        super(SubjectVocabulary, self).__init__(terms, *interfaces)

    def search(self, query_string):
        return [v for v in self if query_string.lower() in v.value.lower()]

    def get_terms(self):
        catalog = api.portal.get_tool('portal_catalog')
        normalizer = queryUtility(IIDNormalizer)
        terms = []

        for term in catalog.uniqueValuesFor('Subject'):
            # Just the normalized term is not enough for an unique token.
            # The unique values are case sensitive. After normalizing the
            # term we get id's in lower-case. That means, we lose the
            # unique value after normalizing if we get values like: "Bond"
            # and "bonD". Adding an md5 hash to the normalized term we
            # prevent double tokens and the token is still readable.
            token = '{0}-{1}'.format(md5(term).hexdigest(),
                                     normalizer.normalize(term))
            terms.append(SimpleTerm(value=term.decode('utf8'),
                                    token=token.decode('utf8'),
                                    title=term.decode('utf8')))
        return terms

directlyProvides(SubjectVocabulary, IVocabularyFactory)
